/* 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 * USA.
 */
/** 
	 * Notificacion.java
	 * Autor: Alberto Poblacin Navarro
	 * Aadido al proyecto en la version v0.2.5
	 * Aadido a:
	 		* Proyecto original: http://code.google.com/p/android-vnc-viewer/
	 		* Release: 39
	 */
package android.androidVNC;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;

public class Notificacion {
	
	//Clase utilizada para mandar las notificaciones
	private NotificationManager nm;
	//Clase que lazar nuestra aplicacin en caso de seleccionar alguna notificacion en la status bar
	private PendingIntent contentIntent;
	//Contexto de la aplicacin
	private Context context;
	//Icono que vamos a dibujar en las notificaciones
	private int icon = R.drawable.icon;
	//Identificador de las notificaciones
	final int ID = 0;
	
	//Aplicamos patrn singleton para asegurarnos que solo haya una instancia de la clase	
	private static Notificacion instancia = null;
	/**
	 * Constructor de instacia de esta clase. Solo se va a crear una instancia por ejecucin del programa.
	 * En caso de que no se haya creado la instacia se crea, en caso contrario se devuelve la creada anteriormente.
	 * @param context contexto de ejecucin del programa
	 * @return instancia de la clase.
	 */
	static public Notificacion Instancia(Context context){
		if (instancia == null)
			instancia = new Notificacion(context);
		return instancia;
	}
	/**
	 * Constructor de la clase que solo se llamar una vez por ejecucin de programa
	 * @param context_ contexto de ejecucin
	 */
	private Notificacion(Context context_) {
		context = context_;
		// Obtemos el servicio para manejar las notificaciones
		nm= (NotificationManager) context.getSystemService(Context.NOTIFICATION_SERVICE);
	    // PendingIntent es un Intent que lanzara nuestra actividad si el usuario seleciona la notifacion
		// en el panel de notificaciones
	    contentIntent = PendingIntent.getActivity(context, 0, new Intent(context,androidVNC.class), 0);
	}
	/**
	 * En caso de llamar a esta funciona la notificacin tendra como ttulo la propia informacin
	 * @param cadena titulo e informacin de la notificacin
	 */
	public void notificar(String cadena){
		notificar(cadena, cadena);
	}
	/**
	 * Funcion que publicar en la barra de estado una notificacin con la informacin
	 * pasada en los parametros de la llamada.
	 * @param titulo Ttulo de la notificacin
	 * @param texto Informacin publicada en la notificacin
	 */
	public void notificar(String titulo, String texto){
		Notification notificacion;
		//Creamos la notificacion y configuramos su configuracin
		notificacion = new Notification(icon, titulo, System.currentTimeMillis());
		notificacion.setLatestEventInfo(context, titulo, texto, contentIntent);
		//Publicamos la notificacin en la barra de estado
		nm.notify(ID, notificacion);
	}
	//Eliminamos las notificaciones de las barras de estado.
	public void eliminar(){
		nm.cancel(ID);
	}
		
}
